from typing import Dict, List
from sqlalchemy.exc import IntegrityError, SQLAlchemyError, DatabaseError as SqlAlchemyDatabaseError
from sqlalchemy.orm import joinedload
from sqlalchemy import delete
from utils.errors.database import DatabaseError, DuplicateEntryError, DatabaseConnectionError
from models import Book
from database.manager import DatabaseManager
from .author import get_or_create_author
from .book_category import get_or_create_categories
from .book_category_statistics import update_category_statistics

import logging
logger = logging.getLogger(__name__)

def fetch_all_books() -> List[Book]:
    with DatabaseManager.get_session() as session:
        try:
            return session.query(Book) \
                          .options(
                              joinedload(Book.author),
                              joinedload(Book.categories)
                          ) \
                          .all()
        except SqlAlchemyDatabaseError as e:
            logger.critical("Connection with database interrupted")
            raise DatabaseConnectionError("Connection with database interrupted") from e
        except SQLAlchemyError as e:
            logger.error(f"An error occurred when fetching all books: {e}")
            raise DatabaseError("An error occurred when fetching all books") from e

def create_book(book: Dict[str, object], skip_existing: bool = True) -> None:
    create_books([book], skip_existing)

def create_books(books: List[Dict[str, object]], skip_existing: bool = True) -> None:
    try:
        with DatabaseManager.get_session() as session:
            for book in books:
                logger.debug(f"Attempting to create a new book: {book['title']}")

                existing_book = session.query(Book).filter_by(isbn=book["isbn"]).first()

                if existing_book:
                    if skip_existing:
                        logger.warning(f"Book with ISBN {book['isbn']} already exists. Skipping.")
                        continue
                    else:
                        logger.error(f"Book with ISBN {book['isbn']} already exists.")
                        raise DuplicateEntryError(f"Book with ISBN {book['isbn']} already exists.")

                author = get_or_create_author(session, book["author"])
                categories = get_or_create_categories(session, book["categories"])

                new_book = Book(
                    title=book["title"],
                    description=book["description"],
                    year_published=book["year_published"],
                    isbn=book["isbn"],
                    price=book["price"],
                    is_damaged=book["is_damaged"],
                    author=author,
                    categories=categories
                )
                session.add(new_book)

            update_category_statistics(session)
            session.commit()
    except IntegrityError as e:
        logger.warning("Data already exists")
        raise DuplicateEntryError("Data already exists in the database") from e
    except SqlAlchemyDatabaseError as e:
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        logger.error(f"An error occurred when creating the book: {e}")
        raise DatabaseError("An error occurred when creating the book") from e

def update_book(book: Dict[str, object]) -> None:
    try:
        with DatabaseManager.get_session() as session:
            logger.debug(f"Updating book {book['title']}")

            existing_book = session.query(Book).filter_by(isbn=book["isbn"]).first()

            if not existing_book:
                logger.warning(f"Book with ISBN {book['isbn']} not found")
                raise DatabaseError("Book not found in the database")

            author = get_or_create_author(session, book["author"])
            categories = get_or_create_categories(session, book["categories"])
            session.commit()

            existing_book.title = book["title"]
            existing_book.description = book["description"]
            existing_book.year_published = book["year_published"]
            existing_book.isbn = book["isbn"]
            existing_book.price = book["price"]
            existing_book.is_damaged = book["is_damaged"]
            existing_book.status = book["status"]
            existing_book.author = author
            existing_book.categories = categories

            update_category_statistics(session, ignore_config=True)
            session.commit()

            logger.info(f"{book['title']} successfully updated.")
    except IntegrityError as e:
        logger.warning("Data already exists")
        raise DuplicateEntryError("Data already exists in the database") from e
    except SqlAlchemyDatabaseError as e:
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        logger.error(f"An error occurred when updating the book: {e}")
        raise DatabaseError("An error occurred when updating the book") from e

def delete_book(book_id: int) -> None:
    try:
        with DatabaseManager.get_session() as session:
            logger.debug(f"Deleting book id {book_id}")
            stmt = delete(Book).where(Book.id == book_id)
            session.execute(stmt)
            update_category_statistics(session, ignore_config=True)
            session.commit()
            logger.info(f"Successfully deleted book with id {book_id}")
    except SqlAlchemyDatabaseError as e:
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        logger.error(f"An error occurred when updating the book: {e}")
        raise DatabaseError("An error occurred when updating the book") from e

__all__ = ["create_book", "create_books", "update_book", "fetch_all_books", "delete_book"]
