from sqlalchemy import func, insert
from sqlalchemy.orm import Session
from models import BookCategory, BookCategoryStatistics, BookCategoryLink
import logging

logger = logging.getLogger(__name__)

def update_category_statistics(session: Session, ignore_config: bool = False) -> None:
    """
    Updates category statistics by calculating the count of books in each category.

    :param session: SQLAlchemy session object.
    """
    # Fetch book counts per category using a join between book_category and book_category_link
    category_counts = (
        session.query(
            BookCategory.id.label('book_category_id'),
            func.count(BookCategoryLink.book_id).label('book_count')
        )
        .join(BookCategoryLink, BookCategoryLink.book_category_id == BookCategory.id, isouter=True)
        .group_by(BookCategory.id)
        .all()
    )

    # Iterate over the results and update or insert the category statistics
    for category_id, book_count in category_counts:
        # Try to get the existing statistics or create a new one if it doesn't exist
        existing_statistics = (
            session.query(BookCategoryStatistics)
            .filter(BookCategoryStatistics.book_category_id == category_id)
            .one_or_none()
        )

        if existing_statistics:
            # If statistics exist, update the count
            existing_statistics.book_count = book_count
            logger.debug(f"Updated category {category_id} with count {book_count}")
        else:
            # If statistics don't exist, create a new one
            new_statistics = BookCategoryStatistics(
                book_category_id=category_id,
                book_count=book_count
            )
            session.add(new_statistics)
            logger.debug(f"Inserted new statistics for category {category_id} with count {book_count}")

    try:
        # Commit the transaction
        session.commit()
        logger.debug("Category statistics updated successfully")
    except Exception as e:
        # In case of error, rollback the transaction
        logger.error(f"An error occurred while updating category statistics: {e}")
        session.rollback()

__all__ = ["update_category_statistics"]
