import logging

from sqlalchemy.orm import sessionmaker, Session
from sqlalchemy import create_engine, text

from sqlalchemy.exc import DatabaseError

from utils.config import DatabaseConfig, UserConfig
from utils.errors.database import DatabaseConnectionError


class DatabaseManager():

    _instance: 'DatabaseManager' = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self) -> None:
        self.logger = logging.getLogger(__name__)
        self.logger.info("Reading database config")
        database_config = DatabaseConfig()
        user_config = UserConfig()
        self.engine = create_engine('mysql+mysqlconnector://%s:%s@%s:%s/%s' % (
            database_config.user,
            database_config.password,
            database_config.host,
            database_config.port,
            database_config.name),
            pool_pre_ping=True,
            isolation_level=user_config.transaction_level.value)
        self.test_connection()
        self.Session = sessionmaker(bind=self.engine)

    def cleanup(self) -> None:
        self.logger.debug("Closing connection")
        self.engine.dispose()

    def test_connection(self):
        self.logger.debug("Testing database connection")
        try:
            with self.engine.connect() as connection:
                connection.execute(text("select 1"))
            self.logger.debug("Database connection successful")
        except DatabaseError as e:
            self.logger.critical(f"Database connection failed: {e}")
            raise DatabaseConnectionError("Database connection failed") from e

    @classmethod
    def get_session(cls) -> Session:
        user_config = UserConfig()

        # Get the transaction level as a string (e.g., "READ COMMITTED", "SERIALIZABLE")
        isolation_level = user_config.transaction_level.value

        # Create a session with the appropriate transaction isolation level
        session = cls._instance.Session()
        session.connection(execution_options={"isolation_level": isolation_level})
        return session

__all__ = ["DatabaseManager"]