import logging
from typing import List, Dict

from sqlalchemy.exc import IntegrityError, SQLAlchemyError, DatabaseError as SqlAlchemyDatabaseError
from sqlalchemy import delete

from utils.errors.database import DatabaseError, DuplicateEntryError, DatabaseConnectionError
from models import Member
from database.manager import DatabaseManager

logger = logging.getLogger(__name__)

def fetch_all_members() -> List[Member]:
    """
    Fetches all members from the database.

    :return: A list of all members in the database.
    :raises DatabaseConnectionError: If the connection to the database is interrupted.
    :raises DatabaseError: If any other error occurs while fetching members.
    """
    with DatabaseManager.get_session() as session:
        try:
            return session.query(Member).all()
        except SqlAlchemyDatabaseError as e:
            logger.critical("Connection with database interrupted")
            raise DatabaseConnectionError("Connection with database interrupted") from e
        except SQLAlchemyError as e:
            logger.error(f"An error occurred when fetching all members: {e}")
            raise DatabaseError("An error occurred when fetching all members") from e

def create_member(new_member: Dict[str, str]):
    create_members([new_member])

def create_members(members: List[Dict[str, str]]):
    try:
        with DatabaseManager.get_session() as session:
            for member_dict in members:
                member = Member(
                            first_name=member_dict["first_name"],
                            last_name=member_dict["last_name"],
                            email=member_dict["email"],
                            phone=member_dict["phone_number"]
                            )
                session.add(member)
            
            session.commit()
    except IntegrityError as e:
        session.rollback()
        
        if "email" in str(e.orig):
            logger.warning("Email is already in use")
            raise DuplicateEntryError("Email", "Email is already in use") from e
        elif "phone" in str(e.orig):
            logger.warning("Phone number is already in use")
            raise DuplicateEntryError("Phone number", "Phone number is already in use") from e
        else:
            logger.error("Member exists already in the database")
            raise DatabaseError("Member exists already") from e

    except DatabaseError as e:
        session.rollback()
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        session.rollback()
        logger.error(f"An error occurred when saving member: {e}")
        raise DatabaseError("An error occurred when creating a new member") from e

def update_member(member: Dict[str, str]):
    try:
        with DatabaseManager.get_session() as session:
            logger.debug(f"Editing member {member['first_name']} {member['last_name']}")

            existing_member = session.query(Member).get(member["id"])

            if not existing_member:
                logger.warning(f"Member with ID {member['id']} not found")
                raise DatabaseError("Member not found in database")
            
            existing_member.first_name = member["first_name"]
            existing_member.last_name = member["last_name"]
            existing_member.email = member["email"]
            existing_member.phone = member["phone"]

            session.commit()
    except IntegrityError as e:
        session.rollback()
        
        if "email" in str(e.orig):
            logger.warning("Email is already in use")
            raise DuplicateEntryError("Email", "Email is already in use") from e
        elif "phone" in str(e.orig):
            logger.warning("Phone number is already in use")
            raise DuplicateEntryError("Phone number", "Phone number is already in use") from e
        else:
            logger.error("An error occurred when updating member")
            raise DatabaseError("An error occurred when updating member") from e

    except DatabaseError as e:
        session.rollback()
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        session.rollback()
        logger.error(f"An error occurred when saving member: {e}")
        raise DatabaseError("An error occurred when creating a new member") from e


def delete_member(member_id: int) -> None:
    try:
        with DatabaseManager.get_session() as session:
            stmt = delete(Member).where(Member.id == member_id)
            session.execute(stmt)
            session.commit()
    except SqlAlchemyDatabaseError as e:
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        logger.error(f"An error occurred when deleting member: {e}")
        raise DatabaseError("An error occurred when deleting member") from e

__all__ = ["create_member", "create_members", "fetch_all_members", "delete_member"]
