from typing import List
import logging

import xml.etree.ElementTree as ET
from xml.dom import minidom
from xmlschema import XMLSchema

from utils.errors import (
    NoExportEntityError,
    ExportError,
    ExportFileError,
    InvalidContentsError,
    XsdSchemeNotFoundError,
    ImportError,
)
from models import BookCategoryStatisticsOverview
from assets import asset_manager

from utils.errors import DatabaseConnectionError, DatabaseError

from database import fetch_all_book_category_statistics_overviews_with_count

logger = logging.getLogger(__name__)

def export_to_xml(file_path: str) -> None:
    try:
        category_list, count = fetch_all_book_category_statistics_overviews_with_count()

        if not category_list:
            raise NoExportEntityError("No categories found to export")

        xml = category_statistics_to_xml(category_list, count)

        with open(file_path, "w", encoding="utf-8") as file:
            file.write(xml)
    except OSError as e:
        raise ExportFileError("Failed to save to a file") from e
    except DatabaseConnectionError as e:
        raise ExportError("An error with database occurred. Try again later") from e
    except DatabaseError as e:
        raise ExportError("Unknown error occurred") from e

def category_statistics_to_xml(category_statistics: List[BookCategoryStatisticsOverview], total_count: int):
    root = ET.Element("statistics")

    for statistic in category_statistics:
        category_element = ET.SubElement(root, "category")

        name_element = ET.SubElement(category_element, "name")
        name_element.text = statistic.name

        count_element = ET.SubElement(category_element, "count")
        count_element.text = str(statistic.book_count)

    total_count_element = ET.SubElement(root, "total_category_count")
    total_count_element.text = str(total_count)

    tree_str = ET.tostring(root, encoding="unicode")

    pretty_xml = minidom.parseString(tree_str).toprettyxml(indent=(" " * 4))
    return pretty_xml