import os
import logging
from typing import Optional, List, Dict
import xml.etree.ElementTree as ET
from xml.dom import minidom
from xmlschema import XMLSchema

from utils.errors import (
    NoExportEntityError,
    ExportError,
    ExportFileError,
    InvalidContentsError,
    XsdSchemeNotFoundError,
    ImportError,
)
from models import Book
from database import fetch_all_books, create_books
from assets import asset_manager

logger = logging.getLogger(__name__)

try:
    logger.debug("Loading XSD schema")
    SCHEMA = XMLSchema(asset_manager.get_asset("book_import_scheme.xsd"))
except Exception as e:
    logger.error("Failed to load XSD schema")
    raise XsdSchemeNotFoundError(f"Failed to load XSD schema: {e}")


def export_to_xml(file_path: str) -> None:
    all_books = fetch_all_books()

    if not all_books:
        raise NoExportEntityError("No books found to export")

    xml = books_to_xml(all_books)

    try:
        with open(file_path, "w", encoding="utf-8") as file:
            file.write(xml)
    except OSError as e:
        raise ExportFileError("Failed to save to a file") from e

def save_books(books: List[Dict[str, object]]):
    create_books(books)

def parse_from_xml(file_path: str) -> List[Dict[str, object]]:
    if not SCHEMA.is_valid(file_path):
        raise InvalidContentsError("XML file is not valid according to XSD schema.")

    try:
        tree = ET.parse(file_path)
        root = tree.getroot()

        books = []
        for book_element in root.findall("book"):
            title = book_element.find("title").text
            year_published = book_element.find("year_published").text
            description = book_element.find("description").text
            isbn = book_element.find("isbn").text
            price = float(book_element.find("price").text)
            is_damaged = bool(book_element.find("is_damaged").text)

            # Parse author
            author_element = book_element.find("author")
            author = {
                "first_name": author_element.find("first_name").text,
                "last_name": author_element.find("last_name").text,
            }

            # Parse categories
            category_elements = book_element.find("categories").findall("category")
            categories = [category_element.text for category_element in category_elements]

            # Create a book dictionary
            book = {
                "title" : title,
                "description" : description,
                "year_published" : year_published,
                "isbn" : isbn,
                "author" : author,
                "price": price,
                "is_damaged" : is_damaged,
                "categories" : categories,
            }
            books.append(book)

        return books
    except ET.ParseError as e:
        raise ImportError(f"Failed to parse XML file: {e}")


def books_to_xml(books: List[Book]) -> str:
    root = ET.Element("books")

    for book in books:
        # Create a <book> element
        book_element = ET.SubElement(root, "book")

        # Add <title>
        title_element = ET.SubElement(book_element, "title")
        title_element.text = book.title

        # Add <author>
        author_element = ET.SubElement(book_element, "author")

        # Add <first_name>
        author_first_name_element = ET.SubElement(author_element, "first_name")
        author_first_name_element.text = book.author.first_name

        author_last_name_element = ET.SubElement(author_element, "last_name")
        author_last_name_element.text = book.author.last_name

        # Add <description>
        description_element = ET.SubElement(book_element, "description")
        description_element.text = book.description

        # Add <year_published>
        year_published_element = ET.SubElement(book_element, "year_published")
        year_published_element.text = book.year_published

        # Add <isbn>
        isbn_element = ET.SubElement(book_element, "isbn")
        isbn_element.text = book.isbn

        price_element = ET.SubElement(book_element, "price")
        price_element.text = str(book.price)

        damaged_element = ET.SubElement(book_element, "is_damaged")
        damaged_element.text = str(book.is_damaged).lower()

        # Add <categories>
        categories_element = ET.SubElement(book_element, "categories")
        for category in book.categories:
            category_element = ET.SubElement(categories_element, "category")
            category_element.text = category.name

    # Convert the tree to a string
    tree_str = ET.tostring(root, encoding="unicode")

    # Pretty print the XML
    pretty_xml = minidom.parseString(tree_str).toprettyxml(indent=(" " * 4))
    return pretty_xml


__all__ = ["export_to_xml", "parse_from_xml"]
