from typing import Dict, Callable

import logging

from PySide6.QtWidgets import (
    QDialog, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, QTextEdit, QPushButton, QComboBox, QFormLayout, QMessageBox, QCheckBox
)
from PySide6.QtGui import QRegularExpressionValidator
from PySide6.QtCore import QRegularExpression
from models import Book, BookStatusEnum, BookCategory

from database import update_book, create_book

from utils.errors.database import DatabaseError, DatabaseConnectionError, DuplicateEntryError


class BookEditor(QDialog):
    def __init__(self, book: Book = None, parent=None, refresh_callback: Callable[[Dict[str, object]], None] = None):
        super().__init__(parent)

        self.logger = logging.getLogger(__name__)
        self.create_layout()

        self.refresh_callback = refresh_callback

        if book:
            self.book_id = book.id
            self.logger.debug(f"Editing book {book.title}")
            self.create_new = False
            self.fill_with_existing_data(book)
        else:
            self.logger.debug("Editing a new book")
            self.create_new = True

    def create_layout(self):
        self.setWindowTitle("Books")
        self.setMinimumWidth(400)

        # Create main layout
        layout = QVBoxLayout(self)

        # Form layout for book fields
        form_layout = QFormLayout()

        # Title field
        self.title_input = QLineEdit()
        form_layout.addRow("Title:", self.title_input)

        # Author field
        self.author_label = QLineEdit()
        form_layout.addRow("Author: ", self.author_label)

        # Description field
        self.description_input = QTextEdit()
        form_layout.addRow("Description:", self.description_input)

        # Year published field
        self.year_input = QLineEdit()
        form_layout.addRow("Year Published:", self.year_input)

        # ISBN field
        self.isbn_input = QLineEdit()
        self.isbn_expression = QRegularExpression("(\d{13})|(\d{10})")
        self.isbn_validator = QRegularExpressionValidator(self.isbn_expression)
        self.isbn_input.setValidator(self.isbn_validator)
        form_layout.addRow("ISBN:", self.isbn_input)

        # Categories field
        self.categories_input = QLineEdit()
        form_layout.addRow("Categories: ", self.categories_input)

        # Damage field
        self.damage_input = QCheckBox()
        form_layout.addRow("Damage:", self.damage_input)

        # Price field
        self.price_input = QLineEdit()
        self.price_input.setValidator(QRegularExpressionValidator(QRegularExpression(r"^\d+(\.\d{1,2})?$")))
        form_layout.addRow("Price:", self.price_input)

        # Status field
        self.status_input = QComboBox()
        self.status_input.addItems([status.value for status in BookStatusEnum])
        form_layout.addRow("Status:", self.status_input)

        layout.addLayout(form_layout)

        # Buttons
        button_layout = QHBoxLayout()

        self.save_button = QPushButton("Save")
        self.save_button.clicked.connect(self.save_book)
        button_layout.addWidget(self.save_button)

        self.cancel_button = QPushButton("Discard")
        self.cancel_button.clicked.connect(self.reject)
        button_layout.addWidget(self.cancel_button)

        layout.addLayout(button_layout)

    def fill_with_existing_data(self, book: Book):
        self.title_input.setText(book.title)
        self.description_input.setText(book.description)
        self.year_input.setText(book.year_published)
        self.isbn_input.setText(book.isbn)

        full_author_name = f"{book.author.first_name} {book.author.last_name}"
        self.author_label.setText(full_author_name)

        all_categories = ", ".join(category.name for category in book.categories)
        self.categories_input.setText(all_categories)

        self.damage_input.setChecked(book.is_damaged)
        self.price_input.setText(str(book.price))
        self.status_input.setCurrentText(book.status.value)

    def save_book(self):
        try:
            book_object = self.parse_inputs()

            if self.create_new:
                create_book(book_object, skip_existing=False)
            else:
                book_object["id"] = self.book_id
                update_book(book_object)

            QMessageBox.information(None,
                                    "Success",
                                    "Book updated successfully",
                                    QMessageBox.StandardButton.Ok)

            if self.refresh_callback:
                self.refresh_callback(book_object)

            self.accept()
        except ValueError as e:
            QMessageBox.critical(None,
                                 "Invalid Input",
                                 f"Input validation failed: {e}",
                                 QMessageBox.StandardButton.Ok)
        except DuplicateEntryError as e:
            QMessageBox.critical(None,
                                 "ISBN is already in use",
                                 "The ISBN provided is already in use",
                                 QMessageBox.StandardButton.Ok)
        except DatabaseConnectionError as e:
            QMessageBox.critical(None,
                                 "Failed to save",
                                 "Could not connect to the database",
                                 QMessageBox.StandardButton.Ok)
        except DatabaseError as e:
            QMessageBox.critical(None,
                                 "An error occurred",
                                 f"Could not save the book because of the following error: {e}",
                                 QMessageBox.StandardButton.Ok)

    def parse_inputs(self) -> Dict[str, object]:
        # Title validation
        title = self.title_input.text().strip()
        if not title or len(title) > 100:
            raise ValueError("Title must be non-empty and at most 100 characters long.")

        # Author validation
        author_name = self.author_label.text().strip()
        if not author_name or len(author_name.split()) < 2:
            raise ValueError("Author must include at least a first and last name.")

        # Split author name into first and last names
        author_parts = author_name.split()
        first_name = author_parts[0]
        last_name = " ".join(author_parts[1:])

        # Description validation
        description = self.description_input.toPlainText().strip()
        if not description:
            raise ValueError("Description cannot be empty.")

        # Year published validation
        year_published = self.year_input.text().strip()
        if not year_published.isdigit() or len(year_published) != 4 or int(year_published) < 0:
            raise ValueError("Year published must be a 4-digit positive number.")

        # ISBN validation
        isbn = self.isbn_input.text().strip()
        if not isbn or len(isbn) not in (10, 13):
            raise ValueError("ISBN must be either 10 or 13 characters long.")

        # Categories validation
        category_text = self.categories_input.text().strip()
        categories = [category.strip() for category in category_text.split(",") if category.strip()]
        if not categories:
            raise ValueError("At least one category must be specified.")

        # Damage validation
        damage = self.damage_input.isChecked()

        # Price validation  
        price = self.price_input.text().strip()
        try:
            price = float(price)
            if price < 0:
                raise ValueError("Price must be a non-negative number.")
        except ValueError:
            raise ValueError("Price must be a valid decimal number.")

        # Status validation
        status = self.status_input.currentText()

        # Map parsed values to dictionary format for saving
        return {
            "title": title,
            "author": {
                "first_name": first_name,
                "last_name": last_name
            },
            "description": description,
            "year_published": year_published,
            "isbn": isbn,
            "categories": categories,
            "is_damaged": damage,
            "price": price,
            "status": status
        }

__all__ = ["BookEditor"]
