from PySide6.QtGui import QAction, Qt
from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QLabel, QWidget, QMenu, QSizePolicy, QLayout, QMessageBox, QDialog
from models import BooksOverview, Book, BookStatusEnum
from ui.editor import BookEditor
from database.manager import DatabaseManager
from database import delete_book
from utils.errors import DatabaseConnectionError, DatabaseError

STATUS_TO_COLOR_MAP = {
    BookStatusEnum.available: "#3c702e",
    BookStatusEnum.borrowed: "#702525",
    BookStatusEnum.reserved: "#bc7613"
}


class BookCard(QWidget):
    def __init__(self, book_overview: BooksOverview):
        super().__init__()

        self.book_overview = book_overview

        self.setAttribute(Qt.WidgetAttribute.WA_Hover, True)  # Enable hover events
        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground, True)  # Enable styling for background

        # Set initial stylesheet with hover behavior
        self.setStyleSheet("""
            BookCard:hover {
                background-color: palette(highlight);
            }
        """)

        # Layout setup
        layout = QHBoxLayout(self)
        layout.setSizeConstraint(QLayout.SizeConstraint.SetMinimumSize)
        self.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)

        layout.setContentsMargins(10, 10, 10, 10)
        layout.setSpacing(10)

        # Initialize UI components
        self.title_label = QLabel()
        self.author_label = QLabel()
        self.isbn_label = QLabel()
        self.status_label = QLabel()
        self.price_label = QLabel()
        self.is_damaged_label = QLabel()

        # Left-side content
        left_side = QVBoxLayout()
        layout.addLayout(left_side)
        left_side.addWidget(self.title_label)
        left_side.addWidget(self.author_label)
        left_side.addWidget(self.isbn_label)

        # Right-side content
        right_side = QVBoxLayout()
        layout.addLayout(right_side)
        right_side.addWidget(self.status_label)
        right_side.addWidget(self.price_label)
        right_side.addWidget(self.is_damaged_label)

        self.setLayout(layout)
        self.update_display()

    def update_display(self):
        """Refreshes the display of the book card based on its current data."""
        self.title_label.setText(self.book_overview.title)
        self.title_label.setStyleSheet("font-size: 20px; font-weight: bold;")

        self.author_label.setText("By: " + self.book_overview.author_name)
        self.isbn_label.setText("ISBN: " + (self.book_overview.isbn or "Not Available"))

        self.status_label.setText(str(self.book_overview.status.value.capitalize()))
        self.status_label.setStyleSheet(f"color: {STATUS_TO_COLOR_MAP[self.book_overview.status]}; font-size: 20px; font-weight: bold;")
        self.status_label.setAlignment(Qt.AlignmentFlag.AlignRight)

        self.price_label.setText("Price: " + str(self.book_overview.price))
        self.price_label.setAlignment(Qt.AlignmentFlag.AlignRight)

        self.is_damaged_label.setText("Damaged: " + str(self.book_overview.is_damaged))
        self.is_damaged_label.setAlignment(Qt.AlignmentFlag.AlignRight)

    def contextMenuEvent(self, event):
        context_menu = QMenu(self)

        action_edit_book = context_menu.addAction("Edit Book")
        delete_book_action = context_menu.addAction("Delete Book")
        delete_book_action.triggered.connect(self.delete_book)

        action = context_menu.exec_(self.mapToGlobal(event.pos()))

        if action == action_edit_book:
            self.open_editor()

    def open_editor(self):
        """Opens the BookEditor and updates the card if changes are made."""
        with DatabaseManager.get_session() as session:
            book_id = self.book_overview.id
            book = session.query(Book).filter(Book.id == book_id).one_or_none()

            if book:
                editor = BookEditor(book)
                if editor.exec() == QDialog.DialogCode.Accepted:
                    updated_data = editor.parse_inputs()
                    self.refresh(updated_data)
            else:
                QMessageBox.critical(self,
                                     "Error",
                                     "The book you requested could not be found. Try again later",
                                     QMessageBox.StandardButton.Ok)

    def refresh(self, updated_data):
        """Updates the card's data and refreshes the display."""
        self.book_overview.title = updated_data["title"]
        self.book_overview.author_name = f"{updated_data['author']['first_name']} {updated_data['author']['last_name']}"
        self.book_overview.isbn = updated_data["isbn"]
        self.book_overview.status = BookStatusEnum(updated_data["status"])
        self.book_overview.price = updated_data["price"]
        self.book_overview.is_damaged = updated_data["is_damaged"]
        self.update_display()

    def delete_book(self):
        if not self.make_sure():
            return

        try:
            delete_book(self.book_overview.id)
            self.setVisible(False)
        except DatabaseConnectionError as e:
            QMessageBox.critical(None, "Failed", "Connection with database failed", QMessageBox.StandardButton.Ok)
        except DatabaseError as e:
            QMessageBox.critical(None, "Failed", f"An error occurred when deleting book: {e}", QMessageBox.StandardButton.Ok)

    def make_sure(self) -> bool:
        are_you_sure_box = QMessageBox()
        are_you_sure_box.setIcon(QMessageBox.Question)
        are_you_sure_box.setWindowTitle("Are you sure?")
        are_you_sure_box.setText(f"Are you sure you want to delete {self.book_overview.title}?")
        are_you_sure_box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        are_you_sure_box.setDefaultButton(QMessageBox.No)

        # Show the message box and capture the user's response
        response = are_you_sure_box.exec()

        # Handle the response
        return response == QMessageBox.Yes

__all__ = ["BookCard"]
