from PySide6.QtWidgets import (
    QHBoxLayout, QVBoxLayout, QLabel, QWidget, QMenu, QSizePolicy, QLayout, QMessageBox
)
from PySide6.QtGui import Qt
from models import Member, MemberStatusEnum
from database.manager import DatabaseManager
from database import delete_member
from ui.editor import MemberEditor
from utils.errors import DatabaseConnectionError, DatabaseError

STATUS_TO_COLOR_MAP = {
    MemberStatusEnum.active: "#3c702e",
    MemberStatusEnum.inactive: "#702525"
}


class MemberCard(QWidget):
    def __init__(self, member: Member):
        super().__init__()

        self.member = member

        self.setAttribute(Qt.WidgetAttribute.WA_Hover, True)
        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground, True)

        # Set initial stylesheet with hover behavior
        self.setStyleSheet("""
            MemberCard:hover {
                background-color: palette(highlight);
            }
        """)

        # Layout setup
        layout = QHBoxLayout(self)
        layout.setSizeConstraint(QLayout.SizeConstraint.SetMinimumSize)
        self.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)

        layout.setContentsMargins(10, 10, 10, 10)
        layout.setSpacing(10)

        # Initialize UI components
        self.name_label = QLabel()
        self.email_label = QLabel()
        self.phone_label = QLabel()
        self.status_label = QLabel()
        self.register_date_label = QLabel()

        # Left-side content
        left_side = QVBoxLayout()
        layout.addLayout(left_side)
        left_side.addWidget(self.name_label)
        left_side.addWidget(self.email_label)
        left_side.addWidget(self.phone_label)

        # Right-side content
        right_side = QVBoxLayout()
        layout.addLayout(right_side)
        right_side.addWidget(self.status_label)
        right_side.addWidget(self.register_date_label)

        self.setLayout(layout)
        self.update_display()

    def update_display(self):
        """Refreshes the display of the member card based on its current data."""
        self.name_label.setText(f"{self.member.first_name} {self.member.last_name}")
        self.name_label.setStyleSheet("font-size: 20px; font-weight: bold;")

        self.email_label.setText(f"Email: {self.member.email}")
        self.phone_label.setText(f"Phone: {self.member.phone or 'Not Available'}")

        self.status_label.setText(str(self.member.status.value.capitalize()))
        self.status_label.setStyleSheet(f"color: {STATUS_TO_COLOR_MAP[self.member.status]}; font-size: 20px; font-weight: bold;")
        self.status_label.setAlignment(Qt.AlignmentFlag.AlignRight)

        self.register_date_label.setText(f"Registered: {self.member.register_date}")
        self.register_date_label.setAlignment(Qt.AlignmentFlag.AlignRight)

    def mousePressEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            self.contextMenuEvent(event)
        else:
            super().mousePressEvent(event)

    def contextMenuEvent(self, event):
        context_menu = QMenu(self)

        action_edit_member = context_menu.addAction("Edit Member")
        action_deactivate_member = context_menu.addAction("Deactivate Member")
        action_activate_member = context_menu.addAction("Activate Member")
        context_menu.addSeparator()
        delete_member_action = context_menu.addAction("Delete Member")
        delete_member_action.triggered.connect(self.delete_member)

        if self.member.status == MemberStatusEnum.active:
            action_activate_member.setVisible(False)
        elif self.member.status == MemberStatusEnum.inactive:
            action_deactivate_member.setVisible(False)

        action = context_menu.exec_(self.mapToGlobal(event.pos()))

        if action == action_edit_member:
            editor = MemberEditor(self.member, refresh_callback=self.refresh)
            editor.exec()
        elif action == action_deactivate_member:
            self.update_member_status(MemberStatusEnum.inactive)
        elif action == action_activate_member:
            self.update_member_status(MemberStatusEnum.active)

    def delete_member(self):
        if not self.make_sure():
            return

        try:
            delete_member(self.member.id)
            self.setVisible(False)
        except DatabaseConnectionError as e:
            QMessageBox.critical(None, "Failed", "Connection with database failed", QMessageBox.StandardButton.Ok)
        except DatabaseError as e:
            QMessageBox.critical(None, "Failed", f"An error occurred when deleting member: {e}", QMessageBox.StandardButton.Ok)

    def update_member_status(self, new_status):
        with DatabaseManager.get_session() as session:
            try:
                member = session.query(Member).filter(Member.id == self.member.id).one_or_none()
                if member:
                    member.status = new_status
                    session.commit()
                    QMessageBox.information(self, "Status Updated", f"Member status updated to {new_status.value.capitalize()}.")
                    self.member.status = new_status
                    self.update_display()
                else:
                    QMessageBox.critical(self, "Error", "The member you requested could not be found.", QMessageBox.StandardButton.Ok)
            except Exception as e:
                session.rollback()
                print(e)

    def refresh(self, updated_data):
        """Updates the card's data and refreshes the display."""
        self.member.first_name = updated_data["first_name"]
        self.member.last_name = updated_data["last_name"]
        self.member.email = updated_data["email"]
        self.member.phone = updated_data["phone"]
        self.update_display()

    def make_sure(self) -> bool:
        are_you_sure_box = QMessageBox()
        are_you_sure_box.setIcon(QMessageBox.Question)
        are_you_sure_box.setWindowTitle("Are you sure?")
        are_you_sure_box.setText(f"Are you sure you want to delete {self.member.first_name} {self.member.last_name}?")
        are_you_sure_box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        are_you_sure_box.setDefaultButton(QMessageBox.No)

        response = are_you_sure_box.exec()
        return response == QMessageBox.Yes

__all__ = ["MemberCard"]
